from .niryo_ros import NiryoRos
from .niryo_robot import NiryoRobot
from .niryo_topic import NiryoTopic

from .arm.arm import Arm
from .conveyor.conveyor import Conveyor
from .io.io import IO
from .pick_place.pick_place import PickPlace
from .saved_poses.saved_poses import SavedPoses
from .tool.tool import Tool
from .trajectories.trajectories import Trajectories
from .vision.vision import Vision
from .led_ring.led_ring import LedRing
from .sound.sound import Sound
from .frames.frames import Frames

from .enums import *
from .arm.enums import *
from .conveyor.enums import *
from .io.enums import *
from .saved_poses.enums import *
from .tool.enums import *
from .trajectories.enums import *
from .vision.enums import *
from .led_ring.enums import *
from .sound.enums import *
from .frames.enums import *

from .objects import PoseObject
from .arm.objects import HardwareStatusObject, JointStateObject
from .conveyor.objects import ConveyorInfo
from .io.objects import DigitalPinObject
from .vision.objects import CameraInfo, ImageParameters

import warnings

warnings.warn(
    ('This project is officially marked as deprecated and will no longer be maintained or receive updates starting '
     '1st June 2025. We recommend users migrate to PyNiryo: https://pypi.org/project/pyniryo/'),
    DeprecationWarning,
    stacklevel=2
)
